///////////////////////////////////////////////////////////////////////////////
// FileManager.cpp
// Paul Wilson
// Oxford Metrics, May 2000
///////////////////////////////////////////////////////////////////////////////

#include "FileManager.h"
#include "FileDatagroupSection.h"
#include "DatagroupSectionHandler.h"
#include "ParameterSectionHandler.h"
#include "UnknownSectionHandler.h"
#include <cassert>

VFileManager::VFileManager()
{
	InitStandardHandlers();
}

VFileManager::~VFileManager()
{
	std::vector<VSectionHandler *>::iterator HandlerIt;

	for (HandlerIt = m_pSectionHandlers.begin(); HandlerIt != m_pSectionHandlers.end(); HandlerIt++)
	{
		delete *HandlerIt;
		*HandlerIt = NULL;
	}
}
/*
bool VFileManager::ReadNextSection(VFileContents & a_rContents)
// This function assumes that the file pointer is positioned at the
// start of a section - if this is not the case then behaviour is
// undefined.
{
	VSectionHandler::EParseResult eResult = VSectionHandler::ENoError;
	SLONG SectionLength;
	std::string strSectionName;

	if (ReadSectionHeader(SectionLength, strSectionName))
	{
		// Check for the null section header...
		
		if (SectionLength != 0)
		{
			// Nope... it's a valid section.. farm out 

			VSectionHandler * pHandler = FindHandler(strSectionName);
			VSectionHandler::EParseResult eResult = pHandler->ReadSection(a_rContents, SectionLength);
		}
	}
	else
	{
		// Read error
		eResult = VSectionHandler::EReadError;
	}

	return eResult == VSectionHandler::ENoError;
}
*/
bool VFileManager::ReadSectionBody(VFileContents & a_rContents, SLONG a_SectionLen, std::string a_strSectionName)
{
	VSectionHandler::EParseResult eResult = VSectionHandler::ENoError;
	
	VSectionHandler * pHandler = FindHandler(a_strSectionName);
	eResult = pHandler->ReadSection(a_rContents, a_SectionLen);

	return eResult == VSectionHandler::ENoError;
}

VFileContents * VFileManager::ReadVFile()
{
	assert(m_FileWrapper.IsOpen());

	bool bRetVal = false;

	VFileContents * pNewContents = new VFileContents();

	if (pNewContents)
	{
		// First.. seek to the beginning of the file...

		m_FileWrapper.Seek(0, SEEK_SET);

		// Now read and parse the header

		SSHORT VersionNum;

		if (ReadFileHeader(VersionNum))
		{
			pNewContents->SetVersion(VersionNum);	// should we check this value??

			if ((bRetVal = ReadStaticDataSections(*pNewContents)) == true)
				ReadDynamicData(*pNewContents);
		}

		if (!bRetVal)
		{
			// Parse error

			delete pNewContents;
			pNewContents = NULL;
		}

	}
	else
	{
		// Out of memory
	}

	return pNewContents;
}

void VFileManager::InitStandardHandlers()
{
	VDatagroupSectionHandler * pDatagroupHandler = new VDatagroupSectionHandler(m_FileWrapper);
	VParameterSectionHandler * pParameterHandler = new VParameterSectionHandler(m_FileWrapper);
	VUnknownSectionHandler * pUnknownHandler = new VUnknownSectionHandler(m_FileWrapper);

	RegisterHandler(pDatagroupHandler);
	RegisterHandler(pParameterHandler);
	RegisterDefaultHandler(pUnknownHandler);
}

void VFileManager::RegisterHandler(VSectionHandler * a_pHandler)
{
	assert(a_pHandler != NULL);

	m_pSectionHandlers.insert(m_pSectionHandlers.begin(), a_pHandler);
}

void VFileManager::RegisterDefaultHandler(VSectionHandler * a_pHandler)
{
	assert(a_pHandler != NULL);

	m_pSectionHandlers.push_back(a_pHandler);
}

VSectionHandler * VFileManager::FindHandler(std::string a_strSectionName)
{
	std::vector<VSectionHandler *>::iterator HandlerIt;

	for (HandlerIt = m_pSectionHandlers.begin(); HandlerIt != m_pSectionHandlers.end(); HandlerIt++)
	{
		if ((*HandlerIt)->CanHandle(a_strSectionName))
			return *HandlerIt;
	}

	// There should be a default handler.... where is it?
	assert(false);

	return NULL;
}

bool VFileManager::ReadStaticDataSections(VFileContents & a_rContents)
{
	bool bRetVal;
	
	SLONG SectionLen;
	std::string strSectionName;

	while ( (bRetVal = ReadSectionHeader(SectionLen, strSectionName)) && SectionLen > 0)  
	{		
		bRetVal = ReadSectionBody(a_rContents, SectionLen, strSectionName);
	} 

	return bRetVal;
}

bool VFileManager::ReadFileHeader(SSHORT & a_rVersionNum)
{
	BYTE HeaderBuf[FILE_HEADER_SIZE] = {0};

	if (m_FileWrapper.Read(HeaderBuf, FILE_HEADER_SIZE))
	{
		if (HeaderBuf[0] == 'V' && HeaderBuf[1] == '#')
		{
			memcpy(&a_rVersionNum, &HeaderBuf[2], sizeof(SSHORT));

			return true;
		}
	}

	return false;
}

bool VFileManager::ReadSectionHeader(SLONG & a_rSectionLen, std::string & a_rstrSectionName)
{
	BYTE HeaderBuf[SECTION_HEADER_SIZE] = {0};

	bool bRetVal = m_FileWrapper.Read(HeaderBuf, SECTION_HEADER_SIZE);

	if (bRetVal)
	{
		memcpy(&a_rSectionLen, &HeaderBuf[0], sizeof(SLONG));
		a_rstrSectionName = (char *)&HeaderBuf[sizeof(SLONG)];
	}

	return bRetVal;
}

bool VFileManager::ReadDynamicData(VFileContents & a_rContents)
{
	bool bRetVal = true;
	VFileDynamicRecord * pDynRecord = NULL;

	while (!m_FileWrapper.eof())
	{
		bRetVal = ReadDynamicRecord(a_rContents);
	}
/*
	do
	{
		if (!ReadNextDynamicRecord(pDynRecord))
		{
			// something went wrong...
		}

		pDynRecord->BuildRecordFromBytes();
		a_rContents.AddDynamicRecord(pDynRecord);

		// Attempt to match up the dynamic record with its datagroup record

		SSHORT DatagroupID = pDynRecord->GetDatagroupID();
		VFileSection * pSection = a_rContents.GetSectionByName(DATAGROUP_SECTION);

		if (pSection != NULL)
		{
			VFileDatagroupSection * pDGSection = dynamic_cast<VFileDatagroupSection *>(pSection);

			assert(pDGSection != NULL);

			VFileDatagroupRecord * pTargRecord = pDGSection->FindDatagroupByID(DatagroupID);

			assert(pTargRecord != NULL);

			if (pTargRecord != NULL)
			{
				pTargRecord->AddDynamicRecord(pDynRecord);
			}
		}
	}
	while (pDynRecord != NULL);
*/
	return bRetVal;
}

bool VFileManager::ReadDynamicRecord(VFileContents & a_rContents)
{
	bool bRetVal = false;
	SSHORT RecLen = 0;

	if (m_FileWrapper.Read(&RecLen, sizeof(SSHORT)))
	{
		assert (RecLen > 0);

		if (RecLen > 0)
		{
			BYTE * pByteBuf = new BYTE[RecLen];

			if (pByteBuf != NULL)
			{
				if (m_FileWrapper.Read(pByteBuf, RecLen))
				{
					VFileDynamicRecord * pDynRecord = new VFileDynamicRecord(pByteBuf, RecLen);

					if (pDynRecord != NULL)
					{
						pDynRecord->BuildRecordFromBytes();
						a_rContents.AddDynamicRecord(pDynRecord);
						bRetVal = true;
					}
					else
					{
						// out of memory
					}

				}
				else
				{
					// truncated
				}
			}
			else
			{
				// Out of memory
			}
		}
	}
	else
	{
		// truncated
	}
	
	return bRetVal;
}

bool VFileManager::ReadNextDynamicRecord(VFileDynamicRecord * a_pDynRecord)
// This function assumes that the file pointer is positioned at the
// start of a dynamic record - if this is not the case then behaviour is
// undefined.
{
	bool bRetVal = false;

	SSHORT RecLen = 0;

	if (m_FileWrapper.Read(&RecLen, sizeof(SSHORT)))
	{
		if (RecLen > 0)
		{
			BYTE * pByteBuf = new BYTE[RecLen];

			if (pByteBuf != NULL)
			{
				if (m_FileWrapper.Read(pByteBuf, RecLen))
				{
					a_pDynRecord = new VFileDynamicRecord(pByteBuf, RecLen);

					if (a_pDynRecord == NULL)
					{
						// out of memory
					}
				}
				else
				{
					// out of memory
				}
			}
			else
			{
				// Out of memory
			}
		}
	}
	else
	{
		// truncated
	}

	return bRetVal;
}

void VFileManager::ReadSectionNames(std::vector<std::string> & a_rNames)
{
	assert(m_FileWrapper.IsOpen());

	if (m_FileWrapper.IsOpen())
	{
		// Seek the first section header... it will be immediately after the file header 

		m_FileWrapper.Seek(4, SEEK_SET);

		BYTE Buffer[SECTION_HEADER_SIZE];
		SLONG * pSectionLen = (SLONG *)Buffer;

		do
		{
			if (!m_FileWrapper.Read(Buffer, sizeof(Buffer)))
				// Truncated...
				break;

			std::string strCurName = (char *)&Buffer[SECTION_LEN_SIZE];
			a_rNames.push_back(strCurName);

			m_FileWrapper.Seek(*pSectionLen, SEEK_CUR);
		}
		while(*pSectionLen > 0);
	}
}

bool VFileManager::ReadSection(std::string strName, VFileSection * a_pNewSection)
{
	assert(m_FileWrapper.IsOpen());
	assert(strName != "");

	bool bRetVal = false;
/*
	if (m_FileWrapper.IsOpen() && strName != "")
	{
		if (SectionSeek(strName))
		{
			bRetVal = ReadNextSection(a_pNewSection);
		}
	}
*/
	return bRetVal;
}

bool VFileManager::CreateVFile(std::string a_strFilename)
// If the file already exists, calling this function
// will overwrite it.
{
	if (a_strFilename.rfind(".v") == std::string::npos &&
		a_strFilename.rfind(".V") == std::string::npos)
	{
		// Append the correct extension

		a_strFilename += ".V";
	}

	bool bRetVal = m_FileWrapper.CreateFile(a_strFilename);

	if (bRetVal)
		bRetVal = WriteVFileHeader();

	return bRetVal;
}

bool VFileManager::CreateVFile(const char * a_sxFilename)
{
	return CreateVFile(std::string(a_sxFilename));
}

VExtendedFileWrapper::EFileMode VFileManager::OpenVFile(std::string a_strFilename)
// Call will fail if the file does not exist.
{
	return m_FileWrapper.OpenFile(a_strFilename);
}

void VFileManager::CloseVFile()
{
	m_FileWrapper.CloseFile();
}

void VFileManager::WriteVFile(VFileContents * a_pContents)
{
	assert(m_FileWrapper.IsOpen());
	bool bStatusOk = true;

	// First things first... seek to the start of the file

	m_FileWrapper.Seek(0, SEEK_SET);

	if (WriteVFileHeader())
	{
		// Okay... go through and write out all the sections..

		int NumSections = a_pContents->GetNumSections();

		for (int i = 0; i < NumSections; i++)
		{
			VFileSection * pSection = a_pContents->GetSectionAt(i);

			if ( (bStatusOk = WriteSection(pSection)) == false)
				break;
		}
	}
}

bool VFileManager::WriteVFileHeader()
{
	// Write out the header, and then the version number...

	BYTE FileHeader[FILE_HEADER_SIZE];

	SSHORT VersionNum = CURRENT_FORMAT_VERSION;
	
	memcpy(FileHeader, FILE_HEADER_PREFIX, 2);
	memcpy(&FileHeader[2], &VersionNum, sizeof(SSHORT));

	bool bStatusOk = m_FileWrapper.Write(FileHeader, FILE_HEADER_SIZE);	

	m_FileWrapper.Flush();

	return bStatusOk;
}

bool VFileManager::WriteSection(VFileSection * a_pSection)
// If creating a file... this function will write from the current position.
{
	bool bRetVal = false;

	VSectionHandler * pHandler = FindHandler(a_pSection->GetName());

	assert(pHandler != NULL);

	if (m_FileWrapper.GetStatus() == VExtendedFileWrapper::ECreate)
	{
		// Performing a write

		pHandler->WriteSection(a_pSection);
	}
	else
	{
		// Hmm... could be a write or a rewrite...
		// let's see if we can find the section in the file...

		if (m_FileWrapper.SectionSeek(a_pSection->GetName()))
		{
			// We've found it.. it's a rewrite - plus the file pointer is in the right place

			// err.. a REwrite in here...
		}
		else
		{
			// Not there... just do a straight write. File pointer will be immediately before the 
			// static-data terminator... so we can just write the section...

			pHandler->WriteSection(a_pSection);
		}
	}

	return bRetVal;
}

bool VFileManager::WriteStaticDataTerminator()
{
	BYTE NullSection[SECTION_HEADER_SIZE] = {0};

	return m_FileWrapper.Write(NullSection, SECTION_HEADER_SIZE);
}

bool VFileManager::WriteDynamicRecord(VFileDynamicRecord * a_pDynRec)
{
	// Prepare the record for writing..

	a_pDynRec->BuildBytesFromRecord();

	// Write the rec len field...

	SSHORT RecLen = a_pDynRec->GetLength();

	m_FileWrapper.Write(&RecLen, sizeof(SSHORT));

	// ...and now the record.

	const BYTE * pBuffer = a_pDynRec->GetDataBuffer();

	return m_FileWrapper.Write(pBuffer, RecLen);
}

void VFileManager::WriteFlush()
{
	m_FileWrapper.Flush();
}

// This bit is dodgy really.. shouldn't really allow it.. but it's useful for testing.

#ifdef _DEBUG
void VFileManager::Flush()
{
	m_FileWrapper.Flush();
}
#endif

