///////////////////////////////////////////////////////////////////////////////
// ParameterSectionHandler.cpp
// Paul Wilson
// Oxford Metrics, May 2000
///////////////////////////////////////////////////////////////////////////////

#include "ParameterSectionHandler.h"
#include "FileParameterSection.h"
#include <cassert>

VParameterSectionHandler::VParameterSectionHandler(VExtendedFileWrapper & a_rFileWrapper) 
: VSectionHandler(a_rFileWrapper)
{

}

VParameterSectionHandler::~VParameterSectionHandler()
{

}

bool VParameterSectionHandler::CanHandle(std::string a_strSectionName)
{
	return strcasecmp(a_strSectionName.c_str(), PARAMETER_SECTION) == 0;
}

VSectionHandler::EParseResult VParameterSectionHandler::ReadSection(VFileContents & a_rContents, SLONG a_SectionLength)
{
	VFileParameterSection * pNewSection = new VFileParameterSection(a_SectionLength);

	if (pNewSection != NULL)
	{
		SSHORT RecLen = 0;

		if (m_rFile.Read(&RecLen, sizeof(SSHORT)))
		{
			// loop until we find the Section terminator

			while (RecLen != 0)
			{
				VFileParameterRecord * pNewRecord = NULL;
				VSectionHandler::EParseResult eResult;

				if ((eResult = ReadRecord(pNewRecord, RecLen)) == ENoError)
				{
					pNewRecord->BuildRecordFromBytes();
					pNewSection->AddRecord(pNewRecord);
				}
				else if (eResult == EReadError)
				{	
					// Okay.. that record was the last one (and it was incomplete).
					// Have to make do...

					break;
				}
				else if (eResult == EOutOfMemory)
				{
					// This is a fairly serious problem... better cleanup

					delete pNewSection;
					return EOutOfMemory;
				}
				else
				{
					// Er... should be ONE of these
					assert(false);
				}

				if (!m_rFile.Read(&RecLen, sizeof(SSHORT)))
					return EReadError;
			}

			// Right.. we've read all the records... now find out how big the 
			// trailing buffer is and skip it...

			int NumToSkip = a_SectionLength - pNewSection->GetDataLength();
			m_rFile.Seek(NumToSkip, SEEK_CUR);

		}
		else
			return EReadError;
	}
	else
		return EOutOfMemory;

	return ENoError;
}

VSectionHandler::EParseResult VParameterSectionHandler::ReadRecord(VFileParameterRecord * a_pNewRecord, SSHORT a_RecLen)
{
	// Check that the return parameter is NULL.. otherwise memory leaks will occur
	assert(a_pNewRecord == NULL);

	VSectionHandler::EParseResult eRetVal = ENoError;
	BYTE * pNewBuffer = new BYTE[a_RecLen];

	if (pNewBuffer != NULL)
	{
		if (m_rFile.Read(pNewBuffer, a_RecLen))
		{
			a_pNewRecord = new VFileParameterRecord(pNewBuffer, a_RecLen);

			if (a_pNewRecord == NULL)
			{
				eRetVal = EOutOfMemory;
				delete [] pNewBuffer;
			}
		}
		else
		{
			eRetVal = EReadError;

			// Half a record aint no use to us...	

			delete [] pNewBuffer;
		}
	}
	else
	{
		eRetVal = EOutOfMemory;
	}

	return eRetVal;
}

void VParameterSectionHandler::WriteSection(VFileSection * a_pSection)
{

}

bool VParameterSectionHandler::WriteSectionHeader(SLONG a_SectionLen, std::string a_strName)
{
	assert(false);
	return false;
}

bool VParameterSectionHandler::RewriteSection(VFileSection * a_pSection)
{
	assert(false);
	return false;
}

