///////////////////////////////////////////////////////////////////////////////
// FileParameterRecord.h
// Paul Wilson
// Oxford Metrics, May 2000
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <string>
#include <vector>

#include "FileRecord.h"

class VFileParameterRecord : public VFileRecord
{
private:
	std::string			m_strName;
	EDataType			m_Type;	
	BYTE * 				m_pDataValues; 
	std::vector<SSHORT>	m_Dimensions;

	int GetDataTypeWidth() const;
	void SetDataAt(void * a_pData, int a_NumBytes, int a_Index);

public:
	VFileParameterRecord(BYTE * a_RawData, int a_Length);
	virtual ~VFileParameterRecord();

	// Gets

	std::string GetName() const { return m_strName; }
	EDataType GetType() const	{ return m_Type; }
	const std::vector<SSHORT> & GetDimensions() { return m_Dimensions; }

	bool		GetBoolData(int a_Index) const;
	BYTE		GetBinaryData(int a_Index) const;
	SSHORT		GetShortData(int a_Index) const;
	SLONG		GetLongData(int a_Index) const;
	float		GetFloatData(int a_Index) const;
	double		GetDoubleData(int a_Index) const;
	std::string	GetTextData(int a_Index) const;

	int	Get2dIndex(int a_First, int a_Second) const;
	int Get3dIndex(int a_First, int a_Second, int a_Third) const;
	int GetNumDataEntries() const;

	// Sets

	void SetName(std::string a_strName);
	void SetType(EDataType a_Type);
	void DeleteDimension(int a_Index);
	void AddDimension(SSHORT a_NewDimension);
	void InsertDimension(SSHORT a_NewDimension, int a_Index);
	void SetDimensionAt(SSHORT a_NewDimension, int a_Index);

	void SetBoolData(int a_Index, bool a_Value);
	void SetBinaryData(int a_Index, BYTE a_Value);
	void SetShortData(int a_Index, SSHORT a_Value);
	void SetLongData(int a_Index, SLONG a_Value);
	void SetFloatData(int a_Index, float a_Value);
	void SetDoubleData(int a_Index, double a_Value);
	void SetTextData(int a_Index, std::string a_Value);

	virtual int CalculateLength() const;

	virtual void BuildRecordFromBytes();
	virtual SSHORT BuildBytesFromRecord();
};

