#include "ToLocal.h"


#include <maya/MSelectionList.h>
#include <maya/MGlobal.h>
#include <maya/MDistance.h>
#include <maya/MFnDependencyNode.h>
#include <maya/MPlugArray.h>
#include <maya/MFnAnimCurve.h>
#include <maya/MMatrix.h>

#include <iostream>

#include "SpaceLocator.h"

VToLocal::VToLocal()
{
}

void * VToLocal::creator()
{ 	
	return new VToLocal(); 
}

MStatus VToLocal::doIt(const MArgList& args)
{
	return redoIt();
}

MStatus VToLocal::redoIt()
{
	int NumNodesLocalisedSuccessfully = 0;
	MSelectionList SelList;
	MStringArray SelStrings;

	MGlobal::getActiveSelectionList(SelList);
	SelList.getSelectionStrings(SelStrings);

	int NumSel = SelStrings.length();

	// There should be a minimum of two objects selected - a 
	// source node and one (or more) target nodes

	if (NumSel >= 2)
	{
		// The last selected node is the source hierarchy

		int ParentPos = NumSel - 1;
		std::string strFutureParent = SelStrings[ParentPos].asChar();

		MObject ParentDagnode;
		SelList.getDependNode(ParentPos, ParentDagnode);

		MFnDependencyNode FnDagNode(ParentDagnode);
		MPlugArray ParentPlugs;
		FnDagNode.getConnections(ParentPlugs);

		int NumParentPlugs = ParentPlugs.length();

		std::vector<std::pair<int, MMatrix> > ParentTransformations;
		std::vector<MFnAnimCurve *> ParentAnimCurves;

		for (int PlugIdx = 0; PlugIdx < NumParentPlugs; PlugIdx++)
		{
			MPlug CurPlug = ParentPlugs[PlugIdx];

			MStatus status;
			MFnAnimCurve * pAnimCurve = new MFnAnimCurve(CurPlug, &status);

			if (status != MS::kSuccess)
				continue;

			ParentAnimCurves.push_back(pAnimCurve);
		}
/*
		std::vector<MFnAnimCurve *>::iterator iParentCurve;

		for (iParentCurve = ParentAnimCurves.begin(); iParentCurve != ParentAnimCurves.end(); iParentCurve++)
		{
			
		}
*/

		for (int i = 0; i < NumSel - 1; i++)
		{
			std::string strSelection = SelStrings[i].asChar();
			std::string strGetParent = "listRelatives -p -f ";	

			MStringArray Parent;
			MGlobal::executeCommand((strGetParent + strSelection).c_str(), Parent);

			std::string strParent = Parent[0].asChar();

//			if (strParent == "")
			{
				// Cannot perform this operation on anything other than
				// global nodes. So.. if there is a parent then don't touch it.

				NumNodesLocalisedSuccessfully++;

				std::string strString = SelStrings[i].asChar();

				MObject dagnode;
				SelList.getDependNode(i, dagnode);

				MFnDependencyNode DagNode(dagnode);
				MPlugArray plugarray;
				DagNode.getConnections(plugarray);

				int NumPlugs = plugarray.length();

				std::vector<MFnAnimCurve *> AnimCurves;

				for (int k = 0; k < NumPlugs; k++)
				{
					MPlug CurPlug = plugarray[k];
					std::string strPlugname = CurPlug.name().asChar();

					MStatus status;
					MFnAnimCurve * pAnimCurve = new MFnAnimCurve(CurPlug, &status);

					if (status != MS::kSuccess)
						continue;

					AnimCurves.push_back(pAnimCurve);
/*
					for (int time = 1; time < 100; time++)
					{
						MTime curtime(time, MTime::kFilm);
//						double value = pAnimCurve.evaluate(curtime);
						pAnimCurve->addKeyframe(curtime, 0, MFnAnimCurve::kTangentLinear, MFnAnimCurve::kTangentLinear);
					}
*/
				}

//				int Numkeyframes = pAnimCurve->numKeyframes();
				
			}
		}

		// Restore original selection

		MGlobal::setActiveSelectionList(SelList);
	}

	return MS::kSuccess;
}
/*
void VToLocal::FindAllChildLocators(std::string a_strTopNode, std::vector<std::string> & a_rChildLocators)
{
	if (IsLocatorName(a_strTopNode) )
	{
		a_rChildLocators.push_back(a_strTopNode);
	}
	else
	{
		// Do recursion for children

		std::string strGetChildren = "listRelatives -c -f ";	

		MStringArray ChildStrings;
		MGlobal::executeCommand((strGetChildren + a_strTopNode).c_str(), ChildStrings);

		int NumChildren = ChildStrings.length();

		for (int i = 0; i < NumChildren; i++)
		{
			std::string strChild = ChildStrings[i].asChar();

			FindAllChildLocators(strChild, a_rChildLocators);
		}
	}	
}

void VToLocal::CopySourceKeysToTargets(std::vector< std::vector<std::string> > & a_rChildLocators)
{
	// The last vector contains the source keys... all the others are targets

	int NumHierarchies = a_rChildLocators.size();
	std::vector<std::string> SourceKeys = *(a_rChildLocators.end() - 1);

	// Strip off the source vector from the end

	a_rChildLocators.erase(a_rChildLocators.end() - 1);

	std::vector<std::string>::iterator iSourceLocator;

	// Some commands we'll be needing... no point constantly making/deleting them

	std::string strCopyCmd = "copyKey ";
	std::string strPasteCmd = "pasteKey -o replace ";

	for (iSourceLocator = SourceKeys.begin(); iSourceLocator != SourceKeys.end(); iSourceLocator++)
	{
		// for each of these source locators... see if there is a matching name in any 
		// of the target hierarchies - if so, copy the keys across

		std::vector< std::vector<std::string> >::iterator iHierarchy;
		std::vector<std::string>::iterator iLocator;

		int SourceScopePos = (*iSourceLocator).rfind("|");
		std::string strSourceBody = (*iSourceLocator).substr(SourceScopePos);

		for (iHierarchy = a_rChildLocators.begin(); iHierarchy != a_rChildLocators.end(); iHierarchy++)
		{
			for (iLocator = (*iHierarchy).begin(); iLocator != (*iHierarchy).end(); iLocator++)
			{
				int DestScopePos = (*iLocator).rfind("|");
				std::string strDestBody = (*iLocator).substr(DestScopePos);

				if (strSourceBody == strDestBody)
				{
					// Found it... do the copy and then remove the locator from our target list
					// to avoid redundant searches...
#ifdef _DEBUG
					std::string strCopyTest = (strCopyCmd + (*iSourceLocator));
					std::string strPasteTest = (strPasteCmd + (*iLocator));
#endif

					MGlobal::executeCommand((strCopyCmd + (*iSourceLocator)).c_str());
					MGlobal::executeCommand((strPasteCmd + (*iLocator)).c_str());

					(*iHierarchy).erase(iLocator);
				}
			}
		}
	}
}

bool VToLocal::IsLocatorName(std::string a_strObjectName)
{
	bool bRetVal = false;

	std::string strGetChildren = "listRelatives -c -f ";	

	MStringArray ChildStrings;
	MGlobal::executeCommand((strGetChildren + a_strObjectName).c_str(), ChildStrings);

	int NumChildren = ChildStrings.length();

	for (int i = 0; i < NumChildren; i++)
	{
		std::string strChild = ChildStrings[i].asChar();

		if (strChild.find(DOF_LOCATOR) != -1 || strChild.find(MARKER_LOCATOR) != -1)
		{
			bRetVal = true;
			break;
		}
	}

	return bRetVal;
}
*/

